from django.test import TestCase 
from django.conf import settings 
from django.core.exceptions import ImproperlyConfigured 

from app_metrics.utils import * 

class MixpanelMetricConfigTests(TestCase): 

    def setUp(self): 
        self.old_backend = settings.APP_METRICS_BACKEND
        settings.APP_METRICS_BACKEND = 'app_metrics.backends.mixpanel'

    def test_metric(self): 
        self.assertRaises(ImproperlyConfigured, metric, 'test_metric')

    def tearDown(self): 
        settings.APP_METRICS_BACKEND = self.old_backend 

class MixpanelCreationTests(TestCase): 

    def setUp(self): 
        self.old_backend = settings.APP_METRICS_BACKEND
        self.old_token = settings.APP_METRICS_MIXPANEL_TOKEN 
        settings.APP_METRICS_BACKEND = 'app_metrics.backends.mixpanel'
        settings.APP_METRICS_MIXPANEL_TOKEN = 'foobar'

    def test_metric(self): 
        metric('testing') 

    def tearDown(self): 
        settings.APP_METRICS_BACKEND = self.old_backend 
        settings.APP_METRICS_MIXPANEL_TOKEN = self.old_token 



from django.core.management.base import NoArgsCommand 

from app_metrics.models import MetricItem
from app_metrics.backends.mixpanel import metric

class Command(NoArgsCommand): 
    help = "Move MetricItems from the db backend to MixPanel" 

    requires_model_validation = True 

    def handle_noargs(self, **options): 
        """ Move MetricItems from the db backend to MixPanel" """ 

        backend = get_backend() 

        # If not using Mixpanel this command is a NOOP
        if backend != 'app_metrics.backends.mixpanel': 
            print "You need to set the backend to MixPanel"
            return 

        items = MetricItem.objects.all() 

        for i in items:
            properties = {
                'time': i.created.strftime('%s'),
            }
            metric(i.metric.slug, num=i.num, properties=properties)

        # Kill off our items 
        items.delete() 



import datetime 

from django.template.loader import render_to_string 

from app_metrics.models import * 
from app_metrics.trending import trending_for_metric 

from django.conf import settings

def generate_report(metric_set=None, html=False): 
    """ Generate a Metric Set Report """ 

    # Get trending data for each metric 
    metric_trends = []
    for m in metric_set.metrics.all(): 
        data = {'metric': m}
        data['trends'] = trending_for_metric(m)
        metric_trends.append(data)

    send_zero_activity = getattr(settings, 'APP_METRICS_SEND_ZERO_ACTIVITY', True)

    if not send_zero_activity:
        activity_today = False
        for trend in metric_trends:
            if trend['trends']['current_day'] > 0:
                activity_today = True
                continue
        if not activity_today:
            return None, None

   
    message = render_to_string('app_metrics/email.txt', { 
                            'metric_set': metric_set, 
                            'metrics': metric_trends, 
                            'today': datetime.date.today(),
                })

    if html: 
        message_html = render_to_string('app_metrics/email.html', { 
                            'metric_set': metric_set, 
                            'metrics': metric_trends, 
                            'today': datetime.date.today(),
                })

        return message, message_html 

    else: 
        return message 



import base64
import json
import urllib
import urllib2

try:
    from celery.task import task
except ImportError:
    from celery.decorators import task

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from app_metrics.models import Metric, MetricItem


class MixPanelTrackError(Exception):
    pass


@task
def db_metric_task(slug, num=1, **kwargs):
    met = Metric.objects.get(slug=slug)
    MetricItem.objects.create(metric=met, num=num)


def _get_token():
    token = getattr(settings, 'APP_METRICS_MIXPANEL_TOKEN', None)

    if not token:
        raise ImproperlyConfigured("You must define APP_METRICS_MIXPANEL_TOKEN when using the mixpanel backend.")
    else:
        return token


@task
def mixpanel_metric_task(slug, num, properties=None, **kwargs):

    token = _get_token()
    if properties == None:
        properties = dict()

    if "token" not in properties:
        properties["token"] = token

    url = getattr(settings, 'APP_METRICS_MIXPANEL_API_URL', "http://api.mixpanel.com/track/")

    params = {"event": slug, "properties": properties}
    b64_data = base64.b64encode(json.dumps(params))

    data = urllib.urlencode({"data": b64_data})
    req = urllib2.Request(url, data)
    for i in range(num):
        response = urllib2.urlopen(req)
        if response.read() == '0':
            raise MixPanelTrackError(u'MixPanel returned 0')



from django.conf.urls.defaults import * 
from django.conf import settings 
from django.contrib import admin 

admin.autodiscover() 

urlpatterns = patterns('', 
        (r'^admin/', include(admin.site.urls)),
        (r'^admin/metrics/', include('app_metrics.urls')),
)



import datetime 
from django.conf import settings 
from django.utils.importlib import import_module 

from app_metrics.models import Metric, MetricSet, MetricItem 

def get_backend(): 
     return getattr(settings, 'APP_METRICS_BACKEND', 'app_metrics.backends.db')

def create_metric_set(name=None, metrics=None, email_recipients=None, no_email=False, send_daily=True, send_weekly=False, send_monthly=False): 
    """ Create a metric set """ 

    # This should be a NOOP for the mixpanel backend 
    backend = get_backend()
    if backend == 'app_metrics.backends.mixpanel': 
        return 

    try: 
        metric_set = MetricSet(
                            name=name, 
                            no_email=no_email, 
                            send_daily=send_daily,
                            send_weekly=send_weekly,
                            send_monthly=send_monthly)
        metric_set.save()

        for m in metrics: 
            metric_set.metrics.add(m)

        for e in email_recipients: 
            metric_set.email_recipients.add(e)

    except: 
        return False 

    return metric_set 

def create_metric(name, slug): 
    """ Create a new type of metric to track """ 

    # Make this a NOOP for the mixpanel backend 
    backend = get_backend() 
    if backend == 'app_metrics.backends.mixpanel': 
        return 

    # See if this metric already exists 
    existing = Metric.objects.filter(name=name, slug=slug) 

    if existing: 
        return False 
    else: 
        new_metric = Metric(name=name, slug=slug)
        new_metric.save()
        return new_metric 

def get_or_create_metric(name, slug):
    """ Returns the metric with the given name and slug, creating it if necessary """

    backend = get_backend() 
    if backend == 'app_metrics.backends.mixpanel': 
        return 
    
    metric, created = Metric.objects.get_or_create(name=name, slug=slug)
    return metric

class InvalidMetricsBackend(Exception): pass 
class MetricError(Exception): pass 

def metric(slug, num=1, **kwargs):
    """ Increment a metric """ 
   
    backend_string = getattr(settings, 'APP_METRICS_BACKEND', 'app_metrics.backends.db')

    # Attempt to import the backend 
    try: 
        backend = import_module(backend_string)
    except: 
        raise InvalidMetricsBackend("Could not load '%s' as a backend" % backend_string )

    try: 
        backend.metric(slug, num, **kwargs)
    except Metric.DoesNotExist: 
        create_metric(slug=slug, name='Autocreated Metric')

def week_for_date(date): 
    return date - datetime.timedelta(days=date.weekday())

def month_for_date(month): 
    return month - datetime.timedelta(days=month.day-1)

def year_for_date(year): 
    return datetime.date(year.year, 01, 01)

def get_previous_month(date): 
    if date.month == 1: 
        month_change = 12 
    else: 
        month_change = date.month - 1 
    new = date 

    return new.replace(month=month_change)

def get_previous_year(date): 
    new = date 
    return new.replace(year=new.year-1)




import datetime 

from django.test import TestCase 
from django.core import management 
from django.conf import settings 
from django.core import mail 
from django.contrib.auth.models import User 

from app_metrics.models import Metric, MetricItem, MetricDay, MetricWeek, MetricMonth, MetricYear
from app_metrics.utils import * 
from app_metrics.trending import _trending_for_current_day, _trending_for_yesterday, _trending_for_week, _trending_for_month, _trending_for_year 

class MetricCreationTests(TestCase): 
   
    def test_metric(self): 
        new_metric = create_metric(name='Test Metric Class',
                                   slug='test_metric')

        metric('test_metric')
        metric('test_metric')
        metric('test_metric')

        current_count = MetricItem.objects.filter(metric=new_metric)
        self.assertEqual(len(current_count), 3)
        self.assertEqual(current_count[0].num, 1)
        self.assertEqual(current_count[1].num, 1)
        self.assertEqual(current_count[2].num, 1)

    def test_get_or_create_metric(self):
        new_metric = get_or_create_metric(name='Test Metric Class',
                                          slug='test_metric')

        metric('test_metric')
        metric('test_metric')
        metric('test_metric')

        new_metric = get_or_create_metric(name='Test Metric Class',
                                          slug='test_metric')

        current_count = MetricItem.objects.filter(metric=new_metric)
        self.assertEqual(len(current_count), 3)
        self.assertEqual(current_count[0].num, 1)
        self.assertEqual(current_count[1].num, 1)
        self.assertEqual(current_count[2].num, 1)

class MetricAggregationTests(TestCase): 

    def setUp(self): 
        self.metric1 = create_metric(name='Test Aggregation1', slug='test_agg1')
        self.metric2 = create_metric(name='Test Aggregation2', slug='test_agg2')

        metric('test_agg1')
        metric('test_agg1')

        metric('test_agg2')
        metric('test_agg2')
        metric('test_agg2')


    def test_daily_aggregation(self): 
        management.call_command('metrics_aggregate')

        day1 = MetricDay.objects.get(metric=self.metric1)
        day2 = MetricDay.objects.get(metric=self.metric2)
        self.assertEqual(day1.num, 2)
        self.assertEqual(day2.num, 3)

    def test_weekly_aggregation(self): 
        management.call_command('metrics_aggregate')

        week1 = MetricWeek.objects.get(metric=self.metric1)
        week2 = MetricWeek.objects.get(metric=self.metric2)
        self.assertEqual(week1.num, 2)
        self.assertEqual(week2.num, 3)

    def test_monthly_aggregation(self): 
        management.call_command('metrics_aggregate')

        month1 = MetricMonth.objects.get(metric=self.metric1)
        month2 = MetricMonth.objects.get(metric=self.metric2)
        self.assertEqual(month1.num, 2)
        self.assertEqual(month2.num, 3)

    def test_yearly_aggregation(self): 
        management.call_command('metrics_aggregate')

        year1 = MetricYear.objects.get(metric=self.metric1)
        year2 = MetricYear.objects.get(metric=self.metric2)
        self.assertEqual(year1.num, 2)
        self.assertEqual(year2.num, 3)

class TrendingTests(TestCase): 
    """ Test that our trending logic works """ 

    def setUp(self): 
        #self.user1 = User.objects.create_user('user1', 'user1@example.com', 'user1pass')
        #self.user2 = User.objects.create_user('user2', 'user2@example.com', 'user2pass')
        self.metric1 = create_metric(name='Test Trending1', slug='test_trend1')
        self.metric2 = create_metric(name='Test Trending2', slug='test_trend2')

    def test_trending_for_current_day(self): 
        """ Test current day trending counter """ 
        metric('test_trend1')
        metric('test_trend1')
        management.call_command('metrics_aggregate')
        metric('test_trend1')
        metric('test_trend1')
        
        count = _trending_for_current_day(self.metric1)
        self.assertEqual(count,4)

    def test_trending_for_yesterday(self): 
        """ Test yesterday trending """ 
        today = datetime.date.today()
        yesterday_date = today - datetime.timedelta(days=1)
        previous_week_date = today - datetime.timedelta(weeks=1)
        previous_month_date = get_previous_month(today)

        MetricDay.objects.create(metric=self.metric1, num=5, created=yesterday_date)
        MetricDay.objects.create(metric=self.metric1, num=4, created=previous_week_date)
        MetricDay.objects.create(metric=self.metric1, num=3, created=previous_month_date)

        data = _trending_for_yesterday(self.metric1)
        self.assertEqual(data['yesterday'], 5)
        self.assertEqual(data['previous_week'], 4)
        self.assertEqual(data['previous_month'], 3)

    def test_trending_for_week(self): 
        """ Test weekly trending data """ 
        this_week_date = week_for_date(datetime.date.today())
        previous_week_date = this_week_date - datetime.timedelta(weeks=1)
        previous_month_date = get_previous_month(this_week_date)
        previous_year_date = get_previous_year(this_week_date)

        MetricWeek.objects.create(metric=self.metric1, num=5, created=this_week_date)
        MetricWeek.objects.create(metric=self.metric1, num=4, created=previous_week_date)
        MetricWeek.objects.create(metric=self.metric1, num=3, created=previous_month_date)
        MetricWeek.objects.create(metric=self.metric1, num=2, created=previous_year_date)
       
        data = _trending_for_week(self.metric1)
        self.assertEqual(data['week'],5)
        self.assertEqual(data['previous_week'],4)
        self.assertEqual(data['previous_month_week'],3)
        self.assertEqual(data['previous_year_week'],2)

    def test_trending_for_month(self): 
        """ Test monthly trending data """ 
        this_month_date = month_for_date(datetime.date.today())
        previous_month_date = get_previous_month(this_month_date)
        previous_month_year_date = get_previous_year(this_month_date)

        MetricMonth.objects.create(metric=self.metric1, num=5, created=this_month_date)
        MetricMonth.objects.create(metric=self.metric1, num=4, created=previous_month_date)
        MetricMonth.objects.create(metric=self.metric1, num=3, created=previous_month_year_date)

        data = _trending_for_month(self.metric1)
        self.assertEqual(data['month'],5)
        self.assertEqual(data['previous_month'],4)
        self.assertEqual(data['previous_month_year'],3)

    def test_trending_for_year(self): 
        """ Test yearly trending data """ 
        this_year_date = year_for_date(datetime.date.today())
        previous_year_date = get_previous_year(this_year_date)

        MetricYear.objects.create(metric=self.metric1, num=5, created=this_year_date)
        MetricYear.objects.create(metric=self.metric1, num=4, created=previous_year_date)

        data = _trending_for_year(self.metric1)
        self.assertEqual(data['year'],5)
        self.assertEqual(data['previous_year'],4)

    def test_missing_trending(self): 
        this_week_date = week_for_date(datetime.date.today())
        previous_week_date = this_week_date - datetime.timedelta(weeks=1)
        previous_month_date = get_previous_month(this_week_date)
        previous_year_date = get_previous_year(this_week_date)

        MetricWeek.objects.create(metric=self.metric1, num=5, created=this_week_date)
        MetricWeek.objects.create(metric=self.metric1, num=4, created=previous_week_date)
        MetricWeek.objects.create(metric=self.metric1, num=3, created=previous_month_date)
       
        data = _trending_for_week(self.metric1)
        self.assertEqual(data['week'],5)
        self.assertEqual(data['previous_week'],4)
        self.assertEqual(data['previous_month_week'],3)
        self.assertEqual(data['previous_year_week'],0)

class EmailTests(TestCase): 
    """ Test that our emails send properly """ 
    def setUp(self): 
        self.user1 = User.objects.create_user('user1', 'user1@example.com', 'user1pass')
        self.user2 = User.objects.create_user('user2', 'user2@example.com', 'user2pass')
        self.metric1 = create_metric(name='Test Trending1', slug='test_trend1')
        self.metric2 = create_metric(name='Test Trending2', slug='test_trend2')
        self.set = create_metric_set(name="Fake Report",
                                     metrics=[self.metric1, self.metric2],
                                     email_recipients=[self.user1, self.user2])

    def test_email(self): 
        """ Test email sending """ 
        metric('test_trend1')
        metric('test_trend1')
        metric('test_trend1')
        metric('test_trend2')
        metric('test_trend2')
        
        management.call_command('metrics_aggregate')
        management.call_command('metrics_send_mail')

        self.assertEqual(len(mail.outbox), 1)




DATABASE_ENGINE='sqlite3'
DATABASE_NAME='test_app_metrics'
SITE_ID = 1
DEBUG = True 

INSTALLED_APPS = [
        'django.contrib.admin',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.sites',
        'app_metrics',
        'app_metrics.tests',
        'djcelery',
]

ROOT_URLCONF = 'app_metrics.tests.urls'

CELERY_ALWAYS_EAGER = True 

APP_METRICS_BACKEND = 'app_metrics.backends.db'
APP_METRICS_MIXPANEL_TOKEN = None 



# Backend to handle sending app metrics directly to mixpanel.com 
# See http://mixpanel.com/api/docs/ for more information on their API 

from django.conf import settings 
from app_metrics.tasks import mixpanel_metric_task
from app_metrics.tasks import _get_token 

def metric(slug, num=1, properties=None): 
    """
    Send metric directly to Mixpanel

    - slug here will be used as the Mixpanel "event" string 
    - if num > 1, we will loop over this and send multiple 
    - properties are a dictionary of additional information you
      may want to pass to Mixpanel.  For example you might use it like: 

      metric("invite-friends",
             properties={"method": "email", "number-friends": "12", "ip": "123.123.123.123"})
    """
    token = _get_token()
    mixpanel_metric_task.delay(slug, num, properties) 




from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response 
from django.template import RequestContext 

@login_required 
def metric_report_view(request):
    return render_to_response('app_metrics/reports.html', {}, context_instance=RequestContext(request))






import datetime 

from django.db import models, IntegrityError
from django.template.defaultfilters import slugify 
from django.contrib.auth.models import User 

class Metric(models.Model): 
    """ The type of metric we want to store """ 
    name    = models.CharField(max_length=50)
    slug    = models.SlugField(unique=True, max_length=60, db_index=True)

    class Meta: 
        verbose_name = 'Metric' 
        verbose_name_plural = 'Metrics' 

    def __unicode__(self): 
        return self.name 

    def save(self, *args, **kwargs): 
        if not self.id and not self.slug: 
            self.slug = slug = slugify(self.name)
            i = 0 
            while True: 
                try: 
                    return super(Metric,self).save(*args, **kwargs)
                except IntegrityError: 
                    i += 1
                    self.slug = "%s_%d" % (self, i)
        else: 
            return super(Metric, self).save(*args, **kwargs)

class MetricSet(models.Model): 
    """ A set of metrics that should be sent via email to certain users """ 
    name          = models.CharField(max_length=50)
    metrics       = models.ManyToManyField(Metric)
    email_recipients = models.ManyToManyField(User)
    no_email      = models.BooleanField(default=False) 
    send_daily    = models.BooleanField(default=True) 
    send_weekly   = models.BooleanField(default=False) 
    send_monthly  = models.BooleanField(default=False) 

    class Meta: 
        verbose_name = 'Metric Set' 
        verbose_name_plural = 'Metric Sets' 

    def __unicode__(self):
        return self.name 

class MetricItem(models.Model): 
    """ Individual metric items """ 
    metric = models.ForeignKey(Metric)
    num  = models.IntegerField(default=1)
    created = models.DateField(default=datetime.datetime.now)

    class Meta: 
        verbose_name = 'Metric Item' 
        verbose_name_plural = 'Metric Items' 

    def __unicode__(self): 
        return "'%s' of %d on %s" % ( self.metric.name, 
                                      self.num, 
                                      self.created )

class MetricDay(models.Model): 
    """ Aggregation of Metrics on a per day basis """ 
    metric  = models.ForeignKey(Metric)
    num   = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today)

    class Meta: 
        verbose_name = 'Day Metric' 
        verbose_name_plural = 'By Day Metrics' 

    def __unicode__(self): 
        return "'%s' for '%s'" % (self.metric.name, self.created)

class MetricWeek(models.Model): 
    """ Aggregation of Metrics on a weekly basis """ 
    metric  = models.ForeignKey(Metric)
    num   = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today)

    class Meta: 
        verbose_name = 'Week Metric' 
        verbose_name_plural = 'By Week Metrics' 

    def __unicode__(self): 
        return "'%s' for week %s of %s" % (self.metric.name, 
                                           self.created.strftime("%U"),
                                           self.created.strftime("%Y"))

class MetricMonth(models.Model): 
    """ Aggregation of Metrics on monthly basis """ 
    metric  = models.ForeignKey(Metric)
    num   = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today)

    class Meta: 
        verbose_name = 'Month Metric' 
        verbose_name_plural = 'By Month Metrics' 

    def __unicode__(self): 
        return "'%s' for %s %s" % (self.metric.name, 
                                   self.created.strftime("%B"), 
                                   self.created.strftime("%Y"))


class MetricYear(models.Model): 
    """ Aggregation of Metrics on a yearly basis """ 
    metric  = models.ForeignKey(Metric)
    num   = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today)

    class Meta: 
        verbose_name = 'Year Metric' 
        verbose_name_plural = 'By Year Metrics' 

    def __unicode__(self): 
        return "'%s' for month of '%s'" % (self.metric.name, 
                                           self.created.strftime("%Y"))





import datetime 
from django.core.management.base import NoArgsCommand 

from app_metrics.models import Metric, MetricItem, MetricDay, MetricWeek, MetricMonth, MetricYear 

from app_metrics.utils import week_for_date, month_for_date, year_for_date, get_backend 

class Command(NoArgsCommand): 
    help = "Aggregate Application Metrics" 

    requires_model_validation = True 

    def handle_noargs(self, **options): 
        """ Aggregate Application Metrics """ 

        backend = get_backend() 

        # If using Mixpanel this command is a NOOP
        if backend == 'app_metrics.backends.mixpanel': 
            print "Useless use of metrics_aggregate when using Mixpanel backend"
            return 

        # Aggregate Items
        items = MetricItem.objects.all() 

        for i in items: 
            # Daily Aggregation 
            day,create = MetricDay.objects.get_or_create(metric=i.metric, 
                                                         created=i.created)

            day.num = day.num + i.num
            day.save() 

            # Weekly Aggregation 
            week_date = week_for_date(i.created)
            week, create = MetricWeek.objects.get_or_create(metric=i.metric,
                                                            created=week_date)

            week.num = week.num + i.num 
            week.save() 

            # Monthly Aggregation 
            month_date = month_for_date(i.created) 
            month, create = MetricMonth.objects.get_or_create(metric=i.metric,
                                                              created=month_date)
            month.num = month.num + i.num 
            month.save() 

            # Yearly Aggregation 
            year_date = year_for_date(i.created) 
            year, create = MetricYear.objects.get_or_create(metric=i.metric,
                                                              created=year_date)
            year.num = year.num + i.num 
            year.save() 

        # Kill off our items 
        items.delete() 
